#pragma config FPLLODIV = DIV_1,FPLLMUL = MUL_20,FPLLIDIV = DIV_2
#pragma config FWDTEN = OFF,FPBDIV = DIV_1,POSCMOD = XT,FNOSC = PRIPLL,CP = OFF
#pragma config FMIIEN = OFF,FETHIO = OFF	
#pragma config UPLLEN = ON,UPLLIDIV = DIV_2

#include "TCPIP Stack/TCPIP.h"
#include "USB/usb.h"
#include "USB/usb_function_hid.h"

APP_CONFIG AppConfig; //   TCP/IP 
BYTE CommonBuf[4];    //   
unsigned char ReceivedDataBuffer[64];  //   USB
unsigned char ToSendDataBuffer[64];    //   USB
USB_HANDLE USBOutHandle = 0;		   //  
USB_HANDLE USBInHandle = 0;			   //  

ROM BYTE SerializedMACAddress[6] =  
	{MY_DEFAULT_MAC_BYTE1,MY_DEFAULT_MAC_BYTE2,MY_DEFAULT_MAC_BYTE3,
 	MY_DEFAULT_MAC_BYTE4,MY_DEFAULT_MAC_BYTE5,MY_DEFAULT_MAC_BYTE6};

void InitAppConfig(void);  //   
void InitializeBoard(void); //   
void ServerTask(void);		//  
void USBTask(void);       // USB 

int main(void)
{
	InitializeBoard(); //  
	TickInit();	      //  
	InitAppConfig();   //  
    StackInit();       //  
    USBDeviceInit();   //  USB Device
    USBDeviceAttach(); //  USB Device


   	while(1)                         //  
    	{
        StackTask();		//   
        StackApplications();    //   
		ServerTask();
		USBTask();              //  USB 
	}
}

void InitializeBoard(void)
{	
	//  LEDs
	LED0_TRIS = 0;
	LED1_TRIS = 0;
	LED2_TRIS = 0;
	LED_PUT(0x00);

	// 
	INTEnableSystemMultiVectoredInt();
		
	//   
	SYSTEMConfigPerformance(GetSystemClock());
	mOSCSetPBDIV(OSC_PB_DIV_1);				
	CNPUESET = 0x00098000; // pullup    
}

void InitAppConfig(void)
{
	memset((void*)&AppConfig, 0x00, sizeof(AppConfig));
	AppConfig.Flags.bIsDHCPEnabled = TRUE;
	AppConfig.Flags.bInConfigMode = TRUE;
	memcpypgm2ram((void*)&AppConfig.MyMACAddr, 
		(ROM void*)SerializedMACAddress, sizeof(AppConfig.MyMACAddr));
	AppConfig.MyIPAddr.Val = MY_DEFAULT_IP_ADDR_BYTE1 | 
		MY_DEFAULT_IP_ADDR_BYTE2<<8ul | MY_DEFAULT_IP_ADDR_BYTE3<<16ul |
		MY_DEFAULT_IP_ADDR_BYTE4<<24ul;
	AppConfig.DefaultIPAddr.Val = AppConfig.MyIPAddr.Val;	
	AppConfig.MyMask.Val = MY_DEFAULT_MASK_BYTE1 | MY_DEFAULT_MASK_BYTE2<<8ul
		| MY_DEFAULT_MASK_BYTE3<<16ul | MY_DEFAULT_MASK_BYTE4<<24ul;
	AppConfig.DefaultMask.Val = AppConfig.MyMask.Val;
	AppConfig.MyGateway.Val = MY_DEFAULT_GATE_BYTE1 | 
		MY_DEFAULT_GATE_BYTE2<<8ul | MY_DEFAULT_GATE_BYTE3<<16ul | 
		MY_DEFAULT_GATE_BYTE4<<24ul;		
	AppConfig.PrimaryDNSServer.Val = MY_DEFAULT_PRIMARY_DNS_BYTE1 | 
		MY_DEFAULT_PRIMARY_DNS_BYTE2<<8ul  | 
		MY_DEFAULT_PRIMARY_DNS_BYTE3<<16ul  | 
		MY_DEFAULT_PRIMARY_DNS_BYTE4<<24ul;
	AppConfig.SecondaryDNSServer.Val = MY_DEFAULT_SECONDARY_DNS_BYTE1 | 
		MY_DEFAULT_SECONDARY_DNS_BYTE2<<8ul  | 
		MY_DEFAULT_SECONDARY_DNS_BYTE3<<16ul  | 
		MY_DEFAULT_SECONDARY_DNS_BYTE4<<24ul;
}

#define SERVER_PORT	9760

void ServerTask(void)
{

	static TCP_SOCKET	MySocket;
	static BOOL isListening = FALSE;
    WORD len;
	BYTE AppBuffer[32];

	if ( !isListening ) {
		MySocket = TCPOpen(0, TCP_OPEN_SERVER, SERVER_PORT, TCP_PURPOSE_GENERIC_TCP_SERVER);
		if(MySocket == INVALID_SOCKET) return;
		isListening = TRUE;
	} else {
		if(!TCPIsConnected(MySocket)) return;
		len = TCPIsGetReady(MySocket);
		if( len>0 ) {
			TCPGetArray(MySocket, AppBuffer, ( len > sizeof(AppBuffer)) ?  sizeof(AppBuffer) : len );
			if (AppBuffer[0] == 'a') {
				LED1_IO = (LED1_IO) ?   0 : 1;
				AppBuffer[0] = BUTTON0_IO;
				AppBuffer[1] = BUTTON1_IO;
				AppBuffer[2] = BUTTON2_IO;
				len = 3;
			}
			else if ( AppBuffer[0] == 'w' ) {
			  if ( len == 5) {
				CommonBuf[0] = AppBuffer[1];
				CommonBuf[1] = AppBuffer[2];
				CommonBuf[2] = AppBuffer[3];
				CommonBuf[3] = AppBuffer[4];	
				AppBuffer[0] = '1';
				AppBuffer[1] = '.';
				AppBuffer[2] = '0';
			  } else {
				AppBuffer[0] = 'E';
				AppBuffer[1] = 'r';
				AppBuffer[2] = 'r';
				len = 3;
			  }			
			} 								
			TCPPutArray(MySocket, AppBuffer, len);
		}

	}
}

BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch(event)
    {
        case EVENT_CONFIGURED: 
			//   EP0 
			USBEnableEndpoint(HID_EP,
				USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|
				USB_DISALLOW_SETUP);
				//   EP0   
				USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
			break;
        case EVENT_EP0_REQUEST:
			USBCheckHIDRequest(); //   HID 
            break;
        default:
            break;
    }      
    return TRUE; 
}

void USBTask(void)
{
    // ,     
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;
    
           // ,      
    if(!HIDRxHandleBusy(USBOutHandle))	    {   
	//  
        switch(ReceivedDataBuffer[0]) //  
        {

            case 'a': //   a.
		   LED2_IO = (LED2_IO) ?   0 : 1; //   
                break;
            case 'r': //  r.
                //       
                ToSendDataBuffer[0] = CommonBuf[0];
                ToSendDataBuffer[1] = CommonBuf[1];
                ToSendDataBuffer[2] = CommonBuf[2];
                ToSendDataBuffer[3] = CommonBuf[3];			
                if(!HIDTxHandleBusy(USBInHandle)) //    
                {
			//  
                    USBInHandle = 
				HIDTxPacket(HID_EP,(BYTE*)&ToSendDataBuffer[0],64);
                }
                break;
        }
        
        //     
        USBOutHandle = HIDRxPacket(HID_EP,(BYTE*)&ReceivedDataBuffer,64);
    }

}
